@@ -126,7 +126,8 @@ class HttpCli(object):
             self.loud_reply(unicode(ex), status=ex.code, volsan=True)
             return self.keepalive
 
-        # time.sleep(0.4)
+        if self.args.rsp_slp:
+            time.sleep(self.args.rsp_slp)
 
         # normalize incoming headers to lowercase;
         # outgoing headers however are Correct-Case
@@ -227,8 +228,8 @@ class HttpCli(object):
         if pwd and "pw" in self.ouparam and pwd != cookies.get("cppwd"):
             self.out_headers["Set-Cookie"] = self.get_pwd_cookie(pwd)[0]
 
-        ua = self.headers.get("user-agent", "")
-        self.is_rclone = ua.startswith("rclone/")
+        self.ua = self.headers.get("user-agent", "")
+        self.is_rclone = self.ua.startswith("rclone/")
         if self.is_rclone:
             uparam["raw"] = False
             uparam["dots"] = False
@@ -283,12 +284,19 @@ class HttpCli(object):
         n = "604800" if cache == "i" else cache or "69"
         self.out_headers["Cache-Control"] = "max-age=" + n
 
+    def k304(self):
+        k304 = self.cookies.get("k304", "")
+        return k304 == "y" or ("; Trident/" in self.ua and not k304)
+
     def send_headers(self, length, status=200, mime=None, headers=None):
         response = ["{} {} {}".format(self.http_ver, status, HTTPCODE[status])]
 
         if length is not None:
             response.append("Content-Length: " + unicode(length))
 
+        if status == 304 and self.k304():
+            self.keepalive = False
+
         # close if unknown length, otherwise take client's preference
         response.append("Connection: " + ("Keep-Alive" if self.keepalive else "Close"))
 
@@ -428,6 +436,9 @@ class HttpCli(object):
             if "ups" in self.uparam:
                 return self.tx_ups()
 
+            if "k304" in self.uparam:
+                return self.set_k304()
+
             if "h" in self.uparam:
                 return self.tx_mounts()
 
@@ -505,7 +516,7 @@ class HttpCli(object):
                 return self.handle_stash()
 
             if "save" in opt:
-                post_sz, _, _, path = self.dump_to_file()
+                post_sz, _, _, _, path = self.dump_to_file()
                 self.log("urlform: {} bytes, {}".format(post_sz, path))
             elif "print" in opt:
                 reader, _ = self.get_body_reader()
@@ -590,8 +601,8 @@ class HttpCli(object):
                 alg = alg or "gz"  # def.pk
                 try:
                     # config-forced opts
-                    alg, lv = pk.split(",")
-                    lv[alg] = int(lv)
+                    alg, nlv = pk.split(",")
+                    lv[alg] = int(nlv)
                 except:
                     pass
 
@@ -621,7 +632,7 @@ class HttpCli(object):
         with ren_open(fn, *open_a, **params) as f:
             f, fn = f["orz"]
             path = os.path.join(fdir, fn)
-            post_sz, _, sha_b64 = hashcopy(reader, f)
+            post_sz, sha_hex, sha_b64 = hashcopy(reader, f)
 
         if lim:
             lim.nup(self.ip)
@@ -645,13 +656,14 @@ class HttpCli(object):
                 time.time(),
             )
 
-        return post_sz, sha_b64, remains, path
+        return post_sz, sha_hex, sha_b64, remains, path
 
     def handle_stash(self):
-        post_sz, sha_b64, remains, path = self.dump_to_file()
+        post_sz, sha_hex, sha_b64, remains, path = self.dump_to_file()
         spd = self._spd(post_sz)
         self.log("{} wrote {}/{} bytes to {}".format(spd, post_sz, remains, path))
-        self.reply("{}\n{}\n".format(post_sz, sha_b64).encode("utf-8"))
+        m = "{}\n{}\n{}\n".format(post_sz, sha_b64, sha_hex[:56])
+        self.reply(m.encode("utf-8"))
         return True
 
     def _spd(self, nbytes, add=True):
@@ -783,6 +795,10 @@ class HttpCli(object):
         return True
 
     def handle_search(self, body):
+        idx = self.conn.get_u2idx()
+        if not hasattr(idx, "p_end"):
+            raise Pebkac(500, "sqlite3 is not available on the server; cannot search")
+
         vols = []
         seen = {}
         for vtop in self.rvol:
@@ -794,7 +810,6 @@ class HttpCli(object):
             seen[vfs] = True
             vols.append([vfs.vpath, vfs.realpath, vfs.flags])
 
-        idx = self.conn.get_u2idx()
         t0 = time.time()
         if idx.p_end:
             penalty = 0.7
@@ -854,63 +869,63 @@ class HttpCli(object):
         response = x.get()
         chunksize, cstart, path, lastmod = response
 
-        if self.args.nw:
-            path = os.devnull
-
-        if remains > chunksize:
-            raise Pebkac(400, "your chunk is too big to fit")
-
-        self.log("writing {} #{} @{} len {}".format(path, chash, cstart, remains))
-
-        reader = read_socket(self.sr, remains)
-
-        f = None
-        fpool = not self.args.no_fpool
-        if fpool:
-            with self.mutex:
-                try:
-                    f = self.u2fh.pop(path)
-                except:
-                    pass
-
-        f = f or open(fsenc(path), "rb+", 512 * 1024)
-
         try:
-            f.seek(cstart[0])
-            post_sz, _, sha_b64 = hashcopy(reader, f)
+            if self.args.nw:
+                path = os.devnull
 
-            if sha_b64 != chash:
-                raise Pebkac(
-                    400,
-                    "your chunk got corrupted somehow (received {} bytes); expected vs received hash:\n{}\n{}".format(
-                        post_sz, chash, sha_b64
-                    ),
-                )
+            if remains > chunksize:
+                raise Pebkac(400, "your chunk is too big to fit")
 
-            if len(cstart) > 1 and path != os.devnull:
-                self.log(
-                    "clone {} to {}".format(
-                        cstart[0], " & ".join(unicode(x) for x in cstart[1:])
-                    )
-                )
-                ofs = 0
-                while ofs < chunksize:
-                    bufsz = min(chunksize - ofs, 4 * 1024 * 1024)
-                    f.seek(cstart[0] + ofs)
-                    buf = f.read(bufsz)
-                    for wofs in cstart[1:]:
-                        f.seek(wofs + ofs)
-                        f.write(buf)
+            self.log("writing {} #{} @{} len {}".format(path, chash, cstart, remains))
 
-                    ofs += len(buf)
+            reader = read_socket(self.sr, remains)
 
-                self.log("clone {} done".format(cstart[0]))
-        finally:
-            if not fpool:
-                f.close()
-            else:
+            f = None
+            fpool = not self.args.no_fpool
+            if fpool:
                 with self.mutex:
-                    self.u2fh.put(path, f)
+                    try:
+                        f = self.u2fh.pop(path)
+                    except:
+                        pass
+
+            f = f or open(fsenc(path), "rb+", 512 * 1024)
+
+            try:
+                f.seek(cstart[0])
+                post_sz, _, sha_b64 = hashcopy(reader, f)
+
+                if sha_b64 != chash:
+                    m = "your chunk got corrupted somehow (received {} bytes); expected vs received hash:\n{}\n{}"
+                    raise Pebkac(400, m.format(post_sz, chash, sha_b64))
+
+                if len(cstart) > 1 and path != os.devnull:
+                    self.log(
+                        "clone {} to {}".format(
+                            cstart[0], " & ".join(unicode(x) for x in cstart[1:])
+                        )
+                    )
+                    ofs = 0
+                    while ofs < chunksize:
+                        bufsz = min(chunksize - ofs, 4 * 1024 * 1024)
+                        f.seek(cstart[0] + ofs)
+                        buf = f.read(bufsz)
+                        for wofs in cstart[1:]:
+                            f.seek(wofs + ofs)
+                            f.write(buf)
+
+                        ofs += len(buf)
+
+                    self.log("clone {} done".format(cstart[0]))
+            finally:
+                if not fpool:
+                    f.close()
+                else:
+                    with self.mutex:
+                        self.u2fh.put(path, f)
+        finally:
+            x = self.conn.hsrv.broker.put(True, "up2k.release_chunk", ptop, wark, chash)
+            x.get()  # block client until released
 
         x = self.conn.hsrv.broker.put(True, "up2k.confirm_chunk", ptop, wark, chash)
         x = x.get()
@@ -957,15 +972,13 @@ class HttpCli(object):
     def get_pwd_cookie(self, pwd):
         if pwd in self.asrv.iacct:
             msg = "login ok"
-            dt = datetime.utcfromtimestamp(time.time() + 60 * 60 * 24 * 365)
-            exp = dt.strftime("%a, %d %b %Y %H:%M:%S GMT")
+            dur = 60 * 60 * 24 * 365
         else:
             msg = "naw dude"
             pwd = "x"  # nosec
-            exp = "Fri, 15 Aug 1997 01:00:00 GMT"
+            dur = None
 
-        ck = "cppwd={}; Path=/; Expires={}; SameSite=Lax".format(pwd, exp)
-        return [ck, msg]
+        return [gencookie("cppwd", pwd, dur), msg]
 
     def handle_mkdir(self):
         new_dir = self.parser.require("name", 512)
@@ -1073,7 +1086,7 @@ class HttpCli(object):
                         f, fname = f["orz"]
                         abspath = os.path.join(fdir, fname)
                         self.log("writing to {}".format(abspath))
-                        sz, sha512_hex, _ = hashcopy(p_data, f)
+                        sz, sha_hex, sha_b64 = hashcopy(p_data, f)
                         if sz == 0:
                             raise Pebkac(400, "empty files in post")
 
@@ -1086,7 +1099,7 @@ class HttpCli(object):
                             bos.unlink(abspath)
                             raise
 
-                    files.append([sz, sha512_hex, p_file, fname, abspath])
+                    files.append([sz, sha_hex, sha_b64, p_file, fname, abspath])
                     dbv, vrem = vfs.get_dbv(rem)
                     self.conn.hsrv.broker.put(
                         False,
@@ -1138,7 +1151,7 @@ class HttpCli(object):
             jmsg["error"] = errmsg
             errmsg = "ERROR: " + errmsg
 
-        for sz, sha512, ofn, lfn, ap in files:
+        for sz, sha_hex, sha_b64, ofn, lfn, ap in files:
             vsuf = ""
             if self.can_read and "fk" in vfs.flags:
                 vsuf = "?k=" + gen_filekey(
@@ -1149,8 +1162,13 @@ class HttpCli(object):
                 )[: vfs.flags["fk"]]
 
             vpath = "{}/{}".format(upload_vpath, lfn).strip("/")
-            msg += 'sha512: {} // {} bytes // <a href="/{}">{}</a> {}\n'.format(
-                sha512[:56], sz, quotep(vpath) + vsuf, html_escape(ofn, crlf=True), vsuf
+            msg += 'sha512: {} // {} // {} bytes // <a href="/{}">{}</a> {}\n'.format(
+                sha_hex[:56],
+                sha_b64,
+                sz,
+                quotep(vpath) + vsuf,
+                html_escape(ofn, crlf=True),
+                vsuf,
             )
             # truncated SHA-512 prevents length extension attacks;
             # using SHA-512/224, optionally SHA-512/256 = :64
@@ -1160,7 +1178,8 @@ class HttpCli(object):
                     self.headers.get("host", "copyparty"),
                     vpath + vsuf,
                 ),
-                "sha512": sha512[:56],
+                "sha512": sha_hex[:56],
+                "sha_b64": sha_b64,
                 "sz": sz,
                 "fn": lfn,
                 "fn_orig": ofn,
@@ -1378,8 +1397,7 @@ class HttpCli(object):
             if "gzip" not in supported_editions:
                 decompress = True
             else:
-                ua = self.headers.get("user-agent", "")
-                if re.match(r"MSIE [4-6]\.", ua) and " SV1" not in ua:
+                if re.match(r"MSIE [4-6]\.", self.ua) and " SV1" not in self.ua:
                     decompress = True
 
             if not decompress:
@@ -1692,10 +1710,16 @@ class HttpCli(object):
             tagq=vs["tagq"],
             mtpq=vs["mtpq"],
             url_suf=suf,
+            k304=self.k304(),
         )
         self.reply(html.encode("utf-8"))
         return True
 
+    def set_k304(self):
+        ck = gencookie("k304", self.uparam["k304"], 60 * 60 * 24 * 365)
+        self.out_headers["Set-Cookie"] = ck
+        self.redirect("", "?h#cc")
+
     def tx_404(self, is_403=False):
         if self.args.vague_403:
             m = '<h1>404 not found &nbsp;┐( ´ -`)┌</h1><p>or maybe you don\'t have access -- try logging in or <a href="/?h">go home</a></p>'
@@ -1812,13 +1836,16 @@ class HttpCli(object):
         if not self.args.unpost:
             raise Pebkac(400, "the unpost feature is disabled in server config")
 
+        idx = self.conn.get_u2idx()
+        if not hasattr(idx, "p_end"):
+            raise Pebkac(500, "sqlite3 is not available on the server; cannot unpost")
+
         filt = self.uparam.get("filter")
         lm = "ups [{}]".format(filt)
         self.log(lm)
 
         ret = []
         t0 = time.time()
-        idx = self.conn.get_u2idx()
         lim = time.time() - self.args.unpost
         for vol in self.asrv.vfs.all_vols.values():
             cur = idx.get_cur(vol.realpath)
