@@ -104,6 +104,7 @@ MIMES = {
     "txt": "text/plain",
     "js": "text/javascript",
     "opus": "audio/ogg; codecs=opus",
+    "caf": "audio/x-caf",
     "mp3": "audio/mpeg",
     "m4a": "audio/mp4",
     "jpg": "image/jpeg",
@@ -821,6 +822,17 @@ def gen_filekey(salt, fspath, fsize, inode):
     ).decode("ascii")
 
 
+def gencookie(k, v, dur):
+    v = v.replace(";", "")
+    if dur:
+        dt = datetime.utcfromtimestamp(time.time() + dur)
+        exp = dt.strftime("%a, %d %b %Y %H:%M:%S GMT")
+    else:
+        exp = "Fri, 15 Aug 1997 01:00:00 GMT"
+
+    return "{}={}; Path=/; Expires={}; SameSite=Lax".format(k, v, exp)
+
+
 def humansize(sz, terse=False):
     for unit in ["B", "KiB", "MiB", "GiB", "TiB"]:
         if sz < 1024:
@@ -1312,9 +1324,17 @@ def guess_mime(url, fallback="application/octet-stream"):
     return ret
 
 
-def runcmd(argv):
+def runcmd(argv, timeout=None):
     p = sp.Popen(argv, stdout=sp.PIPE, stderr=sp.PIPE)
-    stdout, stderr = p.communicate()
+    if not timeout or PY2:
+        stdout, stderr = p.communicate()
+    else:
+        try:
+            stdout, stderr = p.communicate(timeout=timeout)
+        except sp.TimeoutExpired:
+            p.kill()
+            stdout, stderr = p.communicate()
+
     stdout = stdout.decode("utf-8", "replace")
     stderr = stderr.decode("utf-8", "replace")
     return [p.returncode, stdout, stderr]
