@@ -525,13 +525,15 @@ function Donut(uc, st) {
     }
 
     r.on = function (ya) {
-        r.fc = 99;
+        r.fc = r.tc = 99;
         r.eta = null;
         r.base = pos();
         optab.innerHTML = ya ? svg() : optab.getAttribute('ico');
         el = QS('#ops a .donut');
-        if (!ya)
+        if (!ya) {
             favico.upd();
+            wintitle();
+        }
     };
     r.do = function () {
         if (!el)
@@ -541,6 +543,11 @@ function Donut(uc, st) {
             v = pos() - r.base,
             ofs = el.style.strokeDashoffset = o - o * v / t;
 
+        if (++r.tc >= 10) {
+            wintitle(f2f(v * 100 / t, 1) + '%, ' + r.eta + 's, ', true);
+            r.tc = 0;
+        }
+
         if (favico.txt) {
             if (++r.fc < 10 && r.eta && r.eta > 99)
                 return;
@@ -728,7 +735,6 @@ function up2k_init(subtle) {
         if (++nenters <= 0)
             nenters = 1;
 
-        //console.log(nenters, Date.now(), 'enter', this, e.target);
         if (onover.bind(this)(e))
             return true;
 
@@ -750,12 +756,19 @@ function up2k_init(subtle) {
         ebi('up_dz').setAttribute('err', mup || '');
         ebi('srch_dz').setAttribute('err', msr || '');
     }
+    function onoverb(e) {
+        // zones are alive; disable cuo2duo branch
+        document.body.ondragover = document.body.ondrop = null;
+        return onover.bind(this)(e);
+    }
     function onover(e) {
         try {
             var ok = false, dt = e.dataTransfer.types;
             for (var a = 0; a < dt.length; a++)
                 if (dt[a] == 'Files')
                     ok = true;
+                else if (dt[a] == 'text/uri-list')
+                    return true;
 
             if (!ok)
                 return true;
@@ -781,17 +794,20 @@ function up2k_init(subtle) {
             clmod(ebi('drops'), 'vis');
             clmod(ebi('up_dz'), 'hl');
             clmod(ebi('srch_dz'), 'hl');
+            // cuo2duo:
+            document.body.ondragover = onover;
+            document.body.ondrop = gotfile;
         }
-
-        //console.log(nenters, Date.now(), 'leave', this, e && e.target);
     }
     document.body.ondragenter = ondrag;
     document.body.ondragleave = offdrag;
+    document.body.ondragover = onover;
+    document.body.ondrop = gotfile;
 
     var drops = [ebi('up_dz'), ebi('srch_dz')];
     for (var a = 0; a < 2; a++) {
         drops[a].ondragenter = ondrag;
-        drops[a].ondragover = onover;
+        drops[a].ondragover = onoverb;
         drops[a].ondragleave = offdrag;
         drops[a].ondrop = gotfile;
     }
@@ -801,7 +817,10 @@ function up2k_init(subtle) {
         ev(e);
         nenters = 0;
         offdrag.bind(this)();
-        var dz = (this && this.getAttribute('id'));
+        var dz = this && this.getAttribute('id');
+        if (!dz && e && e.clientY)
+            // cuo2duo fallback
+            dz = e.clientY < window.innerHeight / 2 ? 'up_dz' : 'srch_dz';
 
         var err = this.getAttribute('err');
         if (err)
@@ -1069,7 +1088,7 @@ function up2k_init(subtle) {
     }
     more_one_file();
 
-    var etaref = 0, etaskip = 0, op_minh = 0;
+    var etaref = 0, etaskip = 0, utw_minh = 0;
     function etafun() {
         var nhash = st.busy.head.length + st.busy.hash.length + st.todo.head.length + st.todo.hash.length,
             nsend = st.busy.upload.length + st.todo.upload.length,
@@ -1082,13 +1101,10 @@ function up2k_init(subtle) {
 
         //ebi('acc_info').innerHTML = humantime(st.time.busy) + ' ' + f2f(now / 1000, 1);
 
-        var op = ebi('op_up2k'),
-            uff = ebi('u2footfoot'),
-            minh = QS('#op_up2k.act') ? Math.max(op_minh, uff.offsetTop + uff.offsetHeight - op.offsetTop + 32) : 0;
-
-        if (minh > op_minh || !op_minh) {
-            op_minh = minh;
-            op.style.minHeight = op_minh + 'px';
+        var minh = QS('#op_up2k.act') && st.is_busy ? Math.max(utw_minh, ebi('u2tab').offsetHeight + 32) : 0;
+        if (utw_minh < minh || !utw_minh) {
+            utw_minh = minh;
+            ebi('u2tabw').style.minHeight = utw_minh + 'px';
         }
 
         if (!nhash)
@@ -1211,15 +1227,16 @@ function up2k_init(subtle) {
             running = true;
             while (true) {
                 var now = Date.now(),
-                    is_busy = 0 !=
-                        st.todo.head.length +
-                        st.todo.hash.length +
-                        st.todo.handshake.length +
-                        st.todo.upload.length +
-                        st.busy.head.length +
-                        st.busy.hash.length +
-                        st.busy.handshake.length +
-                        st.busy.upload.length;
+                    oldest_active = Math.min(  // gzip take the wheel
+                        st.todo.head.length ? st.todo.head[0].n : st.files.length,
+                        st.todo.hash.length ? st.todo.hash[0].n : st.files.length,
+                        st.todo.upload.length ? st.todo.upload[0].nfile : st.files.length,
+                        st.todo.handshake.length ? st.todo.handshake[0].n : st.files.length,
+                        st.busy.head.length ? st.busy.head[0].n : st.files.length,
+                        st.busy.hash.length ? st.busy.hash[0].n : st.files.length,
+                        st.busy.upload.length ? st.busy.upload[0].nfile : st.files.length,
+                        st.busy.handshake.length ? st.busy.handshake[0].n : st.files.length),
+                    is_busy = oldest_active < st.files.length;
 
                 if (was_busy && !is_busy) {
                     for (var a = 0; a < st.files.length; a++) {
@@ -1239,7 +1256,7 @@ function up2k_init(subtle) {
                 }
 
                 if (was_busy != is_busy) {
-                    was_busy = is_busy;
+                    st.is_busy = was_busy = is_busy;
 
                     window[(is_busy ? "add" : "remove") +
                         "EventListener"]("beforeunload", warn_uploader_busy);
@@ -1268,7 +1285,7 @@ function up2k_init(subtle) {
 
                         timer.rm(etafun);
                         timer.rm(donut.do);
-                        op_minh = 0;
+                        utw_minh = 0;
                     }
                     else {
                         timer.add(donut.do);
@@ -1320,7 +1337,8 @@ function up2k_init(subtle) {
                 }
 
                 if (st.todo.head.length &&
-                    st.busy.head.length < parallel_uploads) {
+                    st.busy.head.length < parallel_uploads &&
+                    (!is_busy || st.todo.head[0].n - oldest_active < parallel_uploads * 2)) {
                     exec_head();
                     mou_ikkai = true;
                 }
@@ -1843,7 +1861,8 @@ function up2k_init(subtle) {
                 st.bytes.uploaded += cdr - car;
                 t.bytes_uploaded += cdr - car;
             }
-            else if (txt.indexOf('already got that') !== -1) {
+            else if (txt.indexOf('already got that') + 1 ||
+                txt.indexOf('already being written') + 1) {
                 console.log("ignoring dupe-segment error", t);
             }
             else {
@@ -1851,6 +1870,9 @@ function up2k_init(subtle) {
                     xhr.status, t.name) + (txt || "no further information"));
                 return;
             }
+            orz2(xhr);
+        }
+        function orz2(xhr) {
             apop(st.busy.upload, upt);
             apop(t.postlist, npart);
             if (!t.postlist.length) {
@@ -1872,9 +1894,11 @@ function up2k_init(subtle) {
                 if (crashed)
                     return;
 
-                toast.err(9.98, "failed to upload a chunk,\n" + tries + " retries so far -- retrying in 10sec\n\n" + t.name);
+                if (!toast.visible)
+                    toast.warn(9.98, "failed to upload a chunk;\nprobably harmless, continuing\n\n" + t.name);
+
                 console.log('chunkpit onerror,', ++tries, t);
-                setTimeout(do_send, 10 * 1000);
+                orz2(xhr);
             };
             xhr.open('POST', t.purl, true);
             xhr.setRequestHeader("X-Up2k-Hash", t.hash[npart]);
