@@ -7,8 +7,7 @@ if (!window['console'])
 
 
 var is_touch = 'ontouchstart' in window,
-    IPHONE = /iPhone|iPad|iPod/i.test(navigator.userAgent),
-    ANDROID = /android/i.test(navigator.userAgent),
+    IPHONE = is_touch && /iPhone|iPad|iPod/i.test(navigator.userAgent),
     WINDOWS = navigator.platform ? navigator.platform == 'Win32' : /Windows/.test(navigator.userAgent);
 
 
@@ -181,6 +180,7 @@ function ignex(all) {
     if (!all)
         window.onerror = vis_exh;
 }
+window.onerror = vis_exh;
 
 
 function noop() { }
@@ -286,15 +286,19 @@ function crc32(str) {
 
 
 function clmod(el, cls, add) {
+    if (!el)
+        return false;
+
     if (el.classList) {
         var have = el.classList.contains(cls);
         if (add == 't')
             add = !have;
 
-        if (add != have)
-            el.classList[add ? 'add' : 'remove'](cls);
+        if (!add == !have)
+            return false;
 
-        return;
+        el.classList[add ? 'add' : 'remove'](cls);
+        return true;
     }
 
     var re = new RegExp('\\s*\\b' + cls + '\\s*\\b', 'g'),
@@ -305,12 +309,18 @@ function clmod(el, cls, add) {
 
     var n2 = n1.replace(re, ' ') + (add ? ' ' + cls : '');
 
-    if (n1 != n2)
-        el.className = n2;
+    if (!n1 == !n2)
+        return false;
+
+    el.className = n2;
+    return true;
 }
 
 
 function clgot(el, cls) {
+    if (!el)
+        return;
+
     if (el.classList)
         return el.classList.contains(cls);
 
@@ -782,13 +792,18 @@ var timer = (function () {
 var tt = (function () {
     var r = {
         "tt": mknod("div"),
+        "th": mknod("div"),
         "en": true,
         "el": null,
-        "skip": false
+        "skip": false,
+        "lvis": 0
     };
 
+    r.th.innerHTML = '?';
     r.tt.setAttribute('id', 'tt');
+    r.th.setAttribute('id', 'tth');
     document.body.appendChild(r.tt);
+    document.body.appendChild(r.th);
 
     var prev = null;
     r.cshow = function () {
@@ -798,11 +813,25 @@ var tt = (function () {
         prev = this;
     };
 
-    r.show = function () {
-        if (r.skip) {
-            r.skip = false;
+    var tev;
+    r.dshow = function (e) {
+        clearTimeout(tev);
+        if (!r.getmsg(this))
             return;
-        }
+
+        if (Date.now() - r.lvis < 400)
+            return r.show.bind(this)();
+
+        tev = setTimeout(r.show.bind(this), 800);
+        if (is_touch)
+            return;
+
+        this.addEventListener('mousemove', r.move);
+        clmod(r.th, 'act', 1);
+        r.move(e);
+    };
+
+    r.getmsg = function (el) {
         if (QS('body.bbox-open'))
             return;
 
@@ -810,7 +839,16 @@ var tt = (function () {
         if (cfg !== null && cfg != '1')
             return;
 
-        var msg = this.getAttribute('tt');
+        return el.getAttribute('tt');
+    };
+
+    r.show = function () {
+        clearTimeout(tev);
+        if (r.skip) {
+            r.skip = false;
+            return;
+        }
+        var msg = r.getmsg(this);
         if (!msg)
             return;
 
@@ -824,6 +862,7 @@ var tt = (function () {
         if (dir.indexOf('u') + 1) top = false;
         if (dir.indexOf('d') + 1) top = true;
 
+        clmod(r.th, 'act');
         clmod(r.tt, 'b', big);
         r.tt.style.left = '0';
         r.tt.style.top = '0';
@@ -849,14 +888,27 @@ var tt = (function () {
 
     r.hide = function (e) {
         ev(e);
+        clearTimeout(tev);
         window.removeEventListener('scroll', r.hide);
-        clmod(r.tt, 'show');
+
         clmod(r.tt, 'b');
+        clmod(r.th, 'act');
+        if (clmod(r.tt, 'show'))
+            r.lvis = Date.now();
+
         if (r.el)
             r.el.removeEventListener('mouseleave', r.hide);
+
+        if (e && e.target)
+            e.target.removeEventListener('mousemove', r.move);
     };
 
-    if (is_touch && IPHONE) {
+    r.move = function (e) {
+        r.th.style.left = (e.pageX + 12) + 'px';
+        r.th.style.top = (e.pageY + 12) + 'px';
+    };
+
+    if (IPHONE) {
         var f1 = r.show,
             f2 = r.hide,
             q = [];
@@ -882,14 +934,14 @@ var tt = (function () {
 
     r.att = function (ctr) {
         var _cshow = r.en ? r.cshow : null,
-            _show = r.en ? r.show : null,
+            _dshow = r.en ? r.dshow : null,
             _hide = r.en ? r.hide : null,
             o = ctr.querySelectorAll('*[tt]');
 
         for (var a = o.length - 1; a >= 0; a--) {
             o[a].onfocus = _cshow;
             o[a].onblur = _hide;
-            o[a].onmouseenter = _show;
+            o[a].onmouseenter = _dshow;
             o[a].onmouseleave = _hide;
         }
         r.hide();
