@@ -47,12 +47,13 @@ class Cfg(Namespace):
             mtp=[],
             mte="a",
             mth="",
+            textfiles="",
             hist=None,
             no_idx=None,
             no_hash=None,
             js_browser=None,
             css_browser=None,
-            **{k: False for k in "e2d e2ds e2dsa e2t e2ts e2tsr".split()}
+            **{k: False for k in "e2d e2ds e2dsa e2t e2ts e2tsr no_acode".split()}
         )
 
 
